#include "includes.h"
#include "tools.h"
#include "socket.h"
#include "buffer.h"
#include "list.h"

#define DLLEXPORT extern "C" __declspec(dllexport)

CList buffers;
CList sockets;
CList files;
int AddSocket(CSocket*);
int AddBuffer(CBuffer*);
int AddFile(HANDLE);


//Sockets

DLLEXPORT int tcpconnect(char*ip, int port, int mode)
{
	CSocket* sock = new CSocket();
	if(sock->tcpconnect(ip, port, mode))
		return AddSocket(sock);
	delete sock;
	return -1;
}

DLLEXPORT int tcplisten(int port, int max, int mode)
{
	CSocket* sock = new CSocket();
	if(sock->tcplisten(port, max, mode))
		return AddSocket(sock);
	delete sock;
	return -1;
}

DLLEXPORT int tcpaccept(int sockid, int mode)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return -1;
	CSocket*sock2 = sock->tcpaccept(mode);
	if(sock2 != NULL)return AddSocket(sock2);
	return -1;
}

DLLEXPORT char* tcpip(int sockid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return NULL;
	return sock->tcpip();
}

DLLEXPORT int setnagle(int sockid, int value)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return 0;
	if(value)
		sock->setnagle(true);
	else sock->setnagle(false);
	return 1;
}

DLLEXPORT int tcpconnected(int sockid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return 0;
	if(sock->tcpconnected())return 1;
	else return 0;
}

DLLEXPORT int udpconnect(char*ip, int port, int mode)
{
	CSocket* sock = new CSocket();
	if(sock->udpconnect(ip, port, mode))
		return AddSocket(sock);
	delete sock;
	return -1;
}

DLLEXPORT int sendmessage(int sockid, char*ip, int port, int buffid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(sock == NULL)return -1;
	if(buff == NULL)return -2;
	int size = sock->sendmessage(ip, port, buff);
	if(size < 0)return -sock->lasterror();
	return size;
}

DLLEXPORT int receivemessage(int sockid, int len, int buffid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(sock == NULL)return -1;
	if(buff == NULL)return -2;
	int size = sock->receivemessage(len, buff);
	if(size < 0)
	{
		int error = sock->lasterror();
		if(error == 10054)return 0;
		return -error;
	}
	return size;
}

DLLEXPORT int peekmessage(int sockid, int len, int buffid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(sock == NULL)return -1;
	if(buff == NULL)return -2;
	int size = sock->peekmessage(len, buff);
	if(size < 0)
	{
		int error = sock->lasterror();
		if(error == 10054)return 0;
		return -error;
	}
	return size;
}

DLLEXPORT int setformat(int sockid, int mode, char*sep)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return -1;
	return sock->SetFormat(mode, sep);
}

DLLEXPORT char* lastinIP()
{
	return CSocket::lastinIP();
}

DLLEXPORT int lastinPort()
{
	return CSocket::lastinPort();
}

DLLEXPORT int setsync(int sockid, int mode)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return -1;
	sock->setsync(mode);
	return 1;
}

DLLEXPORT int closesock(int sockid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return -1;
	delete sock;
	sockets.set(sockid, NULL);
	return 1;
}

DLLEXPORT int socklasterror(int sockid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return -1;
	return -sock->lasterror();
}

DLLEXPORT char* myhost()
{
	return CSocket::myhost();
}

DLLEXPORT char* getbroadcastaddress(int sockid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return NULL;
	return sock->getbroadcastaddress();
}

DLLEXPORT int compareip(char*ip, char*mask)
{
	return CTools::comparematch(ip, mask);
}

DLLEXPORT int sockexit()
{
	CSocket::SockExit();
	return 1;
}

DLLEXPORT int sockstart()
{
	CSocket::SockStart();
	return 1;
}

DLLEXPORT char* hostip(char*host)
{
	return CSocket::GetIp(host);
}

DLLEXPORT int getsocketid(int sockid)
{
	CSocket*sock = (CSocket*)sockets.item(sockid);
	if(sock == NULL)return -1;
	return sock->sockid;
}


//Buffers

DLLEXPORT int writeint(int val, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	return buff->writeint(val);
}

DLLEXPORT int writefloat(float val, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	return buff->writefloat(val);
}

DLLEXPORT int writestring(char*str, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	return buff->writestring(str);
}

DLLEXPORT int readint(int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	return buff->readint();
}

DLLEXPORT double readfloat(double buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item((int)buffid);
	if(buff == NULL)return 0;
	return buff->readfloat();
}

DLLEXPORT char* readstring(int buffid)
{

	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return NULL;
	return buff->readstring();
}

DLLEXPORT char* readsep(char*sep, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return NULL;
	return buff->readsep(sep);
}

DLLEXPORT int getpos(int pos, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	if(pos == 0)return buff->writepos;
	else return buff->readpos;
}

DLLEXPORT int clearbuffer(int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	buff->clear();
	return 1;
}

DLLEXPORT int buffsize(int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	return buff->count;
}

DLLEXPORT int setpos(int pos, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	buff->readpos = pos;
	buff->writepos = pos;
	return pos;
}

DLLEXPORT double bytesleft(double buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item((int)buffid);
	if(buff == NULL)return 0;
	return buff->bytesleft();
}

DLLEXPORT int createbuffer()
{
	CBuffer* buff = new CBuffer();
	return AddBuffer(buff);
}

DLLEXPORT int freebuffer(int buffid)
{
	if(buffid == 0)return 0; //no deleting default buffer
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	delete buff;
	buffers.set(buffid, NULL);
	return 1;
}

DLLEXPORT int copybuffer(int destinationid, int sourceid)
{
	CBuffer*destination = (CBuffer*)buffers.item(destinationid);
	if(destination == NULL)return 0;
	CBuffer*source = (CBuffer*)buffers.item(sourceid);
	if(source == NULL)return 0;
	destination->addBuffer(source);
	return 1;

}

DLLEXPORT int copybuffer2(int destinationid, int start, int len, int sourceid)
{
	CBuffer*destination = (CBuffer*)buffers.item(destinationid);
	if(destination == NULL)return 0;
	CBuffer*source = (CBuffer*)buffers.item(sourceid);
	if(source == NULL)return 0;
	destination->addBuffer(source->data+start, len);
	return 1;

}


//Utilities

DLLEXPORT int strcompare(char*str1, char*str2)
{
	return strcmp(str1, str2);
}

DLLEXPORT char* getmacaddress()
{
	return CTools::getmacaddress();
}

DLLEXPORT char* md5string(char*str)
{
	return CTools::md5string(str);
}

DLLEXPORT char* md5buffer(int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return NULL;
	return CTools::md5buffer(buff);
}

DLLEXPORT int bufferencrypt(char*pass, int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	CTools::encryptedbuffer(buff, pass);
	return 1;
}

DLLEXPORT double adler32(double buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item((int)buffid);
	if(buff == NULL)return 0;
	return CTools::adler32(buff);
}

DLLEXPORT int bufferexists(int buffid)
{
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return 0;
	return 1;
}

DLLEXPORT int netconnected()
{
	return CTools::netconnected();
}

DLLEXPORT int fileopen(char*name, int mode)
{
	HANDLE file = CTools::BinOpen(name, mode);
	if(file != NULL)
		return AddFile(file);
	return -1;
}

DLLEXPORT int fileclose(int fileid)
{
	HANDLE file = (HANDLE)files.item(fileid);
	if(file == NULL)return -1;
	files.set(fileid, NULL);
	return CTools::BinClose(file);
}

DLLEXPORT int filewrite(int fileid, int buffid)
{
	HANDLE file = (HANDLE)files.item(fileid);
	if(file == NULL)return -1;
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return -1;
	return CTools::BinWrite(file, buff);
}

DLLEXPORT int fileread(int fileid, int bytes, int buffid)
{
	HANDLE file = (HANDLE)files.item(fileid);
	if(file == NULL)return -1;
	CBuffer*buff = (CBuffer*)buffers.item(buffid);
	if(buff == NULL)return -1;
	return CTools::BinRead(file, bytes, buff);
}

DLLEXPORT int filepos(int fileid)
{
	HANDLE file = (HANDLE)files.item(fileid);
	if(file == NULL)return -1;
	return CTools::BinGetPos(file);
}

DLLEXPORT int filesetpos(int fileid, int pos)
{
	HANDLE file = (HANDLE)files.item(fileid);
	if(file == NULL)return -1;
	return CTools::BinSetPos(file, pos);
}

DLLEXPORT int filesize(int fileid)
{
	HANDLE file = (HANDLE)files.item(fileid);
	if(file == NULL)return -1;
	return CTools::BinFileSize(file);
}

DLLEXPORT int iptouint(char*ip)
{
	return (int)CTools::ip2uint(ip);
}

DLLEXPORT char* uinttoip(int ip)
{
	return CTools::uint2ip((unsigned int)ip);
}

int AddBuffer(CBuffer*b)
{
	for(int i = 0; i < buffers.count; i ++)
	{
		if(buffers[i] == NULL)
		{
			buffers.set(i, b);
			return i;
		}
	}
	buffers.Add(b);
	return buffers.count-1;
}

int AddSocket(CSocket*b)
{
	for(int i = 0; i < sockets.count; i ++)
	{
		if(sockets[i] == NULL)
		{
			sockets.set(i, b);
			return i;
		}
	}
	sockets.Add(b);
	return sockets.count-1;
}

int AddFile(HANDLE b)
{
	for(int i = 0; i < files.count; i ++)
	{
		if(files[i] == NULL)
		{
			files.set(i, b);
			return i;
		}
	}
	files.Add(b);
	return files.count-1;
}

DLLEXPORT double dllInit()
{
    CSocket::SockStart();
    buffers.Add(new CBuffer());
    sockets.Add(new CSocket()); //dummy socket
    return 1;
}

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
    {
        case DLL_PROCESS_DETACH:
			CSocket::SockExit();
			for(int i = 0; i < buffers.count; i++)
				if(buffers[i] != NULL)
					delete (CBuffer*)buffers.item(i);
			for(int i = 0; i < sockets.count; i++)
				if(sockets[i] != NULL)
					delete (CSocket*)sockets.item(i);
			for(int i = 0; i < files.count; i ++)
				if(files[i] != NULL)
					CTools::BinClose((HANDLE)files.item(i));
            break;
    }
    return TRUE;
}
